@@ -1,4 +1,6 @@
 #!/bin/sh
+# SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+# SPDX-FileCopyrightText: 2018 Philip Chimento <philip.chimento@gmail.com>
 
 if test "$GJS_USE_UNINSTALLED_FILES" = "1"; then
     gjs="$TOP_BUILDDIR/cjs-console"
@@ -15,14 +17,15 @@ THE_DIFF=$("$gjs" -d "$JS_SCRIPT" < "$DEBUGGER_SCRIPT" | sed \
     -e "s#$1#$(basename $1)#g" \
     -e "s/0x[0-9a-f]\{4,16\}/0xADDR/g" \
     | diff -u "$EXPECTED_OUTPUT" -)
-if test $? -ne 0; then
-    echo "not ok 1 - $1  # command failed"
-    exit 1
-fi
+EXITCODE=$?
 
 if test -n "$THE_DIFF"; then
     echo "not ok 1 - $1"
     echo "$THE_DIFF" | while read line; do echo "#$line"; done
 else
+    if test $EXITCODE -ne 0; then
+        echo "not ok 1 - $1  # command failed"
+        exit 1
+    fi
     echo "ok 1 - $1"
 fi
