@@ -1,3 +1,7 @@
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2013 Red Hat, Inc.
+// SPDX-FileCopyrightText: 2015 Endless Mobile, Inc.
+
 const {GIMarshallingTests, Gio, GLib, GObject} = imports.gi;
 
 const Foo = GObject.registerClass({
@@ -153,14 +157,75 @@ describe('logError', function () {
         }
     });
 
-    it('logs a SyntaxError', function () {
-        GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
-            'JS ERROR: SyntaxError:*');
-        try {
+    describe('Syntax Error', function () {
+        function throwSyntaxError() {
             Reflect.parse('!@#$%^&');
-        } catch (e) {
-            logError(e);
         }
+
+        it('logs a SyntaxError', function () {
+            GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+                'JS ERROR: SyntaxError:*');
+            try {
+                throwSyntaxError();
+            } catch (e) {
+                logError(e);
+            }
+        });
+
+        it('logs a stack trace with the SyntaxError', function () {
+            GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+                'JS ERROR: SyntaxError:*throwSyntaxError@*');
+            try {
+                throwSyntaxError();
+            } catch (e) {
+                logError(e);
+            }
+        });
+    });
+
+    it('logs an error with cause', function marker() {
+        GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+            'JS ERROR: Error: an error\nmarker@*Caused by: Gio.IOErrorEnum: another error\nmarker2@*');
+        function marker2() {
+            return new Gio.IOErrorEnum({message: 'another error', code: 0});
+        }
+        logError(new Error('an error', {cause: marker2()}));
+    });
+
+    it('logs a GError with cause', function marker() {
+        GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+            'JS ERROR: Gio.IOErrorEnum: an error\nmarker@*Caused by: Error: another error\nmarker2@*');
+        function marker2() {
+            return new Error('another error');
+        }
+        const e = new Gio.IOErrorEnum({message: 'an error', code: 0});
+        e.cause = marker2();
+        logError(e);
+    });
+
+    it('logs an error with non-object cause', function () {
+        GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+            'JS ERROR: Error: an error\n*Caused by: 3');
+        logError(new Error('an error', {cause: 3}));
+    });
+
+    it('logs an error with a cause tree', function () {
+        GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+            'JS ERROR: Error: one\n*Caused by: Error: two\n*Caused by: Error: three\n*');
+        const three = new Error('three');
+        const two = new Error('two', {cause: three});
+        logError(new Error('one', {cause: two}));
+    });
+
+    it('logs an error with cyclical causes', function () {
+        // We cannot assert here with GLib.test_expect_message that the * at the
+        // end of the string doesn't match more causes, but at least the idea is
+        // that it shouldn't go into an infinite loop
+        GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+            'JS ERROR: Error: one\n*Caused by: Error: two\n*');
+        const one = new Error('one');
+        one.cause = new Error('two', {cause: one});
+        logError(one);
     });
 });
 
@@ -201,3 +266,17 @@ describe('thrown GError', function () {
         expect(err.code).toEqual(Gio.IOErrorEnum.NOT_FOUND);
     });
 });
+
+describe('GError.new_literal', function () {
+    it('constructs a valid GLib.Error', function () {
+        const e = GLib.Error.new_literal(
+            Gio.IOErrorEnum, Gio.IOErrorEnum.FAILED, 'message');
+        expect(e instanceof GLib.Error).toBeTruthy();
+        expect(e.code).toEqual(Gio.IOErrorEnum.FAILED);
+        expect(e.message).toEqual('message');
+    });
+    it('does not accept invalid domains', function () {
+        expect(() => GLib.Error.new_literal(0, 0, 'message'))
+            .toThrowError(/0 is not a valid domain/);
+    });
+});
